package Socialtext::WikiTest;
use strict;
use warnings;

=head1 NAME

Socialtext::WikiTest - Execute tests defined on wiki pages

=cut

our $VERSION = '0.07';

=head1 AUTHOR

Luke Closs, C<< <luke.closs at socialtext.com> >>

=head1 COPYRIGHT & LICENSE

Copyright 2006,2007 Luke Closs, all rights reserved.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=cut

1;
